"""
__version__.py
~~~~~~~~~~~~~~

"""

__title__ = 'UC2 REST'
__description__ = 'This pacage will help you to drive the ESP32-driven microscopy control modules from UC2'
__version__ = "v0.2.0.8"
__author__ = 'Benedict Diederich'
__author_email__ = 'benedictdied@gmail.com'
__license__ = 'GPL v3'
__url__ = 'https://github.com/openUC2/UC2-REST'
